const gulp = require('gulp'),
    fs = require('fs'),
    flatten = require('gulp-flatten'),
    phpspec = require('gulp-phpspec'),
    run = require('gulp-run'),
    notify = require('gulp-notify'),
    sass = require('gulp-sass'),
    util = require('gulp-util'),
    concat = require('gulp-concat'),
    uglify = require('gulp-uglify'),
    rename = require('gulp-rename'),
    minifyCss = require('gulp-clean-css'),
    browserify = require('gulp-browserify'),
    zip = require('gulp-zip'),
    git = require('gulp-git'),
    bump = require('gulp-bump'),
    runSequence = require('run-sequence'),
    sourcemaps = require('gulp-sourcemaps');

// To get fresh version and avoid cached result
let getPackageJson = () => require('./package.json');

const ToolGulp = (function () {
    var jqueryPlugins = function () {
            return srcAsset('/js/jquery/jquery.*.js');
        },
        componentsJs = function () {
            return srcAsset('/js/components/*.js');
        },
        icon = function (icon) {
            return './' + icon;
        },
        phpSrc = function (path) {
            return './src' + path;
        },
        phpSpecTests = function (path) {
            return phpTests('/Spec' + path);
        },
        phpTests = function (path) {
            return './tests' + path;
        },
        settingsJs = function () {
            return srcAsset('/js/settings/*.js');
        },
        srcAsset = function (path) {
            return './assets' + path;
        },
        destAsset = function (path) {
            return './src/assets' + path;
        };

    return {
        destAsset: destAsset,
        icon: icon,
        jqueryPlugins: jqueryPlugins,
        phpSpecTests: phpSpecTests,
        phpSrc: phpSrc,
        srcAsset: srcAsset
    };
})();

const scripts = {
    'bootstrap': 'node_modules/bootstrap-sass/assets/javascripts/bootstrap.js',
    'jqueryMin': 'node_modules/jquery/dist/jquery.min.js',
    'jquery': 'node_modules/jquery/dist/jquery.js'
};

/*
 |--------------------------------------------------------------------------
 | Development and Test tasks
 |--------------------------------------------------------------------------
 */

gulp.task('scss:admin', function () {
    return gulp.src(ToolGulp.srcAsset('/scss/admin.scss'))
        .pipe(sourcemaps.init())
        .pipe(sass().on('error', function (err) {
            util.log(err.message);
            this.emit('end')
        }))
        .pipe(sourcemaps.write())
        .pipe(gulp.dest(ToolGulp.destAsset('/css/')));
});

gulp.task('js:admin', function () {
    return gulp.src([ToolGulp.srcAsset('/js/admin.js')])
        .pipe(concat('admin.js'))
        .pipe(browserify({
            paths: ['./node_modules']
        }))
        .pipe(gulp.dest(ToolGulp.destAsset('/js/')))
});


gulp.task('test:phpspec', function () {
    var options = {
        'verbose': 'v',
        clear: true,
        notify: true
    };

    gulp.src(ToolGulp.phpSpecTests('/**/*.php'))
        .pipe(phpspec('', options))
        .on('error', notify.onError({
            title: 'Ops!',
            message: 'Tests failed!, Look over your code and find the error.',
            icon: ToolGulp.icon('fail.png')
        }))
        .pipe(notify({
            title: 'Success!',
            message: 'All tests have passed.',
            icon: ToolGulp.icon('success.png')
        }));
});

/*
 |--------------------------------------------------------------------------
 | Development and Test tasks
 |--------------------------------------------------------------------------
 */

// Add jquery to the theme
gulp.task('js:jquery', function jQueryTask() {
    return gulp.src(scripts['jqueryMin'])
        .pipe(gulp.dest(ToolGulp.destAsset('/js')))
});

// ### Fonts
// `gulp fonts` - Grabs all the fonts and outputs them in a flattened directory
// structure. See: https://github.com/armed/gulp-flatten
gulp.task('fonts', function fontsTask() {
    return gulp.src(ToolGulp.srcAsset('/fonts/**/*'))
        .pipe(flatten())
        .pipe(gulp.dest(ToolGulp.destAsset('/fonts/')))
});

/*
 |--------------------------------------------------------------------------
 | Staging tasks
 |--------------------------------------------------------------------------
 */

gulp.task('scss-stage:admin', function () {
    return gulp.src(ToolGulp.srcAsset('/scss/admin.scss'))
        .pipe(sass().on('error', util.log))
        .pipe(minifyCss({compatibility: 'ie8'}))
        .pipe(rename({
            suffix: '.min'
        }))
        .pipe(gulp.dest(ToolGulp.destAsset('/css/')));
});

gulp.task('js-stage:admin', function () {
    return gulp.src([ToolGulp.srcAsset('/js/admin.js')])
        .pipe(concat('admin.min.js'))
        .pipe(browserify({
            paths: ['./node_modules']
        }))
        .pipe(uglify())
        .pipe(gulp.dest(ToolGulp.destAsset('/js/')))
});

/*
 |--------------------------------------------------------------------------
 | Production tasks
 |--------------------------------------------------------------------------
 */

gulp.task('scss-prod:admin', function () {
    return gulp.src(ToolGulp.srcAsset('/scss/admin.scss'))
        .pipe(sass().on('error', util.log))
        .pipe(minifyCss({compatibility: 'ie8'}))
        .pipe(rename({
            suffix: '.min'
        }))
        .pipe(gulp.dest(ToolGulp.destAsset('/css/')));
});

gulp.task('js-prod:admin', function () {
    return gulp.src([ToolGulp.srcAsset('/js/admin.js')])
        .pipe(concat('admin.min.js'))
        .pipe(browserify({paths: ['./node_modules']}))
        .pipe(uglify())
        .pipe(gulp.dest(ToolGulp.destAsset('/js/')))
});

/** ===============
 *  Bump
 *  ===============
 */
gulp.task('bump', function () {
    runSequence('bump:patch-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:patch', function () {
    runSequence('bump:patch-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:hotfix', function () {
    runSequence('bump:patch-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:patch-task', function () {
    return bumpApplication('patch');
});

gulp.task('bump:minor', function () {
    runSequence('bump:minor-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:minor-task', function () {
    return bumpApplication('minor');
});

gulp.task('bump:major', function () {
    runSequence('bump:major-task', 'bumpCommit', 'gitTag');
});
gulp.task('bump:major-task', function () {
    return bumpApplication('major');
});

function bumpApplication(type) {
    type = type === 'minor' || type === 'major' ? type : 'patch';

    return gulp.src([
        'package.json',
        'package-lock.json',
        'composer.json'
    ])
        .pipe(bump({type: type}).on('error', util.log))
        .pipe(gulp.dest('./'));
}


gulp.task('bumpCommit', function () {
    return gulp.src([
        'package.json',
        'package-lock.json',
        'composer.json'
    ])
        .pipe(git.add())
        .pipe(git.commit('Release v' + getPackageJson().version))
        .pipe(gulp.dest('./'));
});

gulp.task('gitTag', function () {
    return git.tag(getPackageJson().version, 'Release v' + getPackageJson().version, function (err) {
        if (err) throw err;
    });
});

/*
 |--------------------------------------------------------------------------
 | Tasks
 |--------------------------------------------------------------------------
 */

gulp.task('dev', ['scss:admin', 'js:admin', 'js:jquery', 'fonts']);
gulp.task('stage', ['scss-stage:admin', 'js-stage:admin', 'js:jquery', 'fonts']);
gulp.task('prod', ['scss-prod:admin', 'js-prod:admin', 'js:jquery', 'fonts']);

gulp.task('default', ['dev']);


/*
 |--------------------------------------------------------------------------
 | Watcher for development
 |--------------------------------------------------------------------------
 */
gulp.task('watch', ['default'], function () {
    var watch = {
        scss: [
            ToolGulp.srcAsset('/scss/**/*.scss')
        ],
        js: [
            ToolGulp.srcAsset('/js/**/*.js')
        ]
    };

    gulp.watch(watch.scss, ['scss:admin']).on('change', function (event) {
        console.log('File ' + event.path + ' was ' + event.type + ', running tasks...');
    });

    gulp.watch(watch.js, ['js:admin']).on('change', function (event) {
        console.log('File ' + event.path + ' was ' + event.type + ', running tasks...');
    });
});

gulp.task('phpspec', ['test:phpspec'], function () {
    var watch = {
        phpspec: [
            ToolGulp.phpSpecTests('/**/*.php'),
            ToolGulp.phpSrc('/**/*.php')
        ]
    };

    gulp.watch(watch.phpspec, ['test:phpspec']).on('change', function (event) {
        console.log('File ' + event.path + ' was ' + event.type + ', running tasks...');
    });
});

gulp.task('dist', function () {
    return gulp.src([
        './**/*',
        '!./node_modules',
        '!./node_modules/**',
        '!./vendor',
        '!./vendor/**',
        '!./dist',
        '!./dist/**',
    ])
        .pipe(zip(getPackageJson().vendor + '-' + getPackageJson().name + '-' + getPackageJson().version + '.zip'))
        .pipe(gulp.dest('dist'));
});
