<?php declare(strict_types=1);

namespace Infomaker\ProjectPlugin\Admin\SettingsPage;

use Infomaker\Everyware\Base\JsonImporter;
use Infomaker\Everyware\Base\Message;
use Infomaker\Everyware\Base\MessageBag;
use Infomaker\Everyware\Base\Models\SettingsParameter;
use Infomaker\Everyware\Support\Collection;
use Infomaker\Everyware\Support\Str;
use Infomaker\Everyware\Twig\View;
use Infomaker\ProjectPlugin\Interfaces\SettingsPageInterface;

class SettingsParametersPage implements SettingsPageInterface
{
    private const SUBMIT_KEY = 'everyware-settings-parameters-submit';

    protected static $slug = 'settings-parameters';

    protected static $title = 'Settings Parameters';

    protected $textDomain = 'everyware_project_plugin_text_domain';

    /** @var JsonImporter */
    protected $jsonImporter;

    /**
     * @var \Infomaker\Everyware\Support\Storage\CollectionDB
     */
    private $collection;

    public function __construct()
    {
        if (is_admin()) {
            add_action('wp_ajax_everyware_delete_setting_parameter', [$this, 'ajax_delete_setting_parameter']);
        }
    }

    public function getTabSlug(): string
    {
        return Str::slug(static::$slug);
    }

    public function getTabTitle(): string
    {
        return __(static::$title, $this->textDomain);
    }

    public function onPageLoad(): void
    {
        $this->jsonImporter = new JsonImporter();

        $this->collection = SettingsParameter::collection();

        if ($this->isUpdated()) {
            $this->collection->setCollection($this->getSubmittedParameters())->save();
        }
    }

    protected function isUpdated(): bool
    {
        return isset($_POST[static::SUBMIT_KEY]);
    }

    protected function getSubmittedParameters(): Collection
    {
        $submittedParameters = $_POST['fields'] ?? [];
        $validParameters = [];
        foreach ($submittedParameters as $field => $values) {
            foreach (array_filter($values) as $index => $value) {
                if ( ! isset($validParameters[$index])) {
                    $validParameters[$index] = [];
                }

                $validParameters[$index][$field] = $value;
            }
        }

        return SettingsParameter::makeCollection($validParameters);
    }

    public function getStatusMessage(): string
    {
        if ($this->isUpdated()) {
            Message::create('Success', 'Updated! Changes have been saved.');
        } elseif ($this->jsonImporter->isImported()) {
            Message::create('Success', 'Import done! ');
            Message::create('Info', 'Changes have not yet been saved.');
        }

        return MessageBag::messagesToHtml();
    }

    public function pageContent(): string
    {
        $parameters = $this->collection->all();
        $importedParameters = SettingsParameter::makeCollection($this->jsonImporter->getJson());

        if ($this->jsonImporter->isImported()) {
            $parameters = $parameters->merge($importedParameters);
        }

        return View::generate('@projectPlugin/admin/settings-page/settings-parameters', [
            'form_action' => $_SERVER['REQUEST_URI'],
            'submit_key' => static::SUBMIT_KEY,
            'text_domain' => $this->textDomain,
            'template' => [
                'key' => ['label' => 'Key', 'value' => ''],
                'value' => ['label' => 'Value', 'value' => ''],
            ],
            'parameters' => $parameters->map(function ($parameter) {
                return [
                    'key' => ['label' => 'Key', 'value' => $parameter->key],
                    'value' => ['label' => 'Value', 'value' => $parameter->value],
                ];
            })->toArray(),
            'json_import' => $this->generateJsonImporter($importedParameters->toJson(),
                $this->collection->all()->toJson()),
        ]);
    }

    /**
     * @param string $import
     * @param string $export
     *
     * @return string
     */
    private function generateJsonImporter(string $import, string $export): string
    {
        return View::generate('@projectPlugin/utilities/import-export', [
            'field_name' => JsonImporter::POST_KEY,
            'import' => $import ?? '',
            'export' => $export ?? '',
        ]);
    }
}
